{$A+,B-,D+,E-,F+,G+,I-,L+,N+,O-,R-,S-,V-}

UNIT Cameras;

INTERFACE

USES Maths,Basis;

VAR Frame:RECORD
            GetMaxX:INTEGER;
            GetMaxXHalf:INTEGER;
            GetMaxY:INTEGER;
            GetMaxYHalf:INTEGER
    END;

    Camera:RECORD
       x,y,z:RealType;
       alpha,beta,gamma:RealType;
       sina,cosa,sinb,cosb,sinc,cosc:RealType
    END;

    screen:RealType;


PROCEDURE SetFrame(maxX,maxY:INTEGER);
(*** Setze Bildgre                                  ***)

PROCEDURE ResizePict(f:RealType);
(*** Vergert/Verkleinert das Bild ***)

PROCEDURE SetCamera(c:Vector3);
(*** Setzt die Position der Kamera.                   ***)


PROCEDURE TurnCamera(a,b,c:RealType);
(*** Dreht die Kamera.                                ***)

PROCEDURE TurnDegCamera(a,b,c:RealType);


PROCEDURE Calc3D(xx,yy:RealType;VAR px,py,pz,rx,ry,rz:RealType);
(*** Berechnet aus der Pixelkoordinate der See-***)
(*** strahlvektor.                             ***)


PROCEDURE Calc3DEye1(xx,yy:RealType;VAR px,py,pz,rx,ry,rz:RealType);
PROCEDURE Calc3DEye2(xx,yy:RealType;VAR px,py,pz,rx,ry,rz:RealType);
(*** Fhrt die entsprechende Berechnung fr    ***)
(*** zwei Augen im Augenabstand                ***)

IMPLEMENTATION

CONST DEye=20;          (* Augenabstand *)
      screenOrg=2000;   (* Entfernung des Bildschirms vom Betrachter.    *)


VAR scrx,scry,scrz:RealType;
    ff:RealType;

PROCEDURE SetFrame(maxX,maxY:INTEGER);
BEGIN
    WITH Frame DO
    BEGIN
        GetMaxX:=maxX;
        GetMaxY:=maxY;
        GetMaxXHalf:=GetMaxX DIV 2;
        GetMaxYHalf:=GetMaxY DIV 2;
        ff:=1023/GetMaxX
    END;

END;


PROCEDURE ResizePict(f:RealType);
BEGIN
    ff:=1/f
END;


PROCEDURE SetCamera(c:Vector3);
BEGIN
    WITH Camera DO
    BEGIN
        x:=c.x;
        y:=c.y;
        z:=c.z
    END
END;


PROCEDURE TurnCamera(a,b,c:RealType);
BEGIN
    WITH Camera DO
    BEGIN
        alpha:=a;
        beta:=b;
        gamma:=c;
        sina:=Sin(a);
        cosa:=Cos(a);
        sinb:=Sin(b);
        cosb:=Cos(b);
        sinc:=Sin(c);
        cosc:=Cos(c);
        scrx:=0;
        scry:=0;
        scrz:=screen;
        Turn3DRight0(scrx,scry,scrz,sina,cosa,sinb,cosb,sinc,cosc)
    END
END;


PROCEDURE TurnDegCamera(a,b,c:RealType);
BEGIN
    TurnCamera(a*PiDiv180,b*PiDiv180,c*PiDiv180)
END;


PROCEDURE Calc3D(xx,yy:RealType;VAR px,py,pz,rx,ry,rz:RealType);
BEGIN
    WITH Camera DO
    BEGIN
        px:=x-scrx;
        py:=y-scry;
        pz:=z-scrz;
        rx:=(xx-Frame.GetMaxXHalf)*4*ff;
        ry:=(-yy+Frame.GetMaxYHalf)*5*ff;
        rz:=screen;
        Turn3DRight0(rx,ry,rz,sina,cosa,sinb,cosb,sinc,cosc)
    END
END;


PROCEDURE Calc3DEye1(xx,yy:RealType;VAR px,py,pz,rx,ry,rz:RealType);
BEGIN
    WITH Camera DO
    BEGIN
        px:=x-scrx-DEye;
        py:=y-scry;
        pz:=z-scrz;
        rx:=(xx+DEye-Frame.GetMaxXHalf)*4*ff;
        ry:=(-yy+Frame.GetMaxYHalf)*5*ff;
        rz:=screen;
        Turn3DRight0(rx,ry,rz,sina,cosa,sinb,cosb,sinc,cosc)
    END
END;


PROCEDURE Calc3DEye2(xx,yy:RealType;VAR px,py,pz,rx,ry,rz:RealType);
BEGIN
    WITH Camera DO
    BEGIN
        px:=x-scrx+DEye;
        py:=y-scry;
        pz:=z-scrz;
        rx:=(xx-DEye-Frame.GetMaxXHalf)*4*ff;
        ry:=(-yy+Frame.GetMaxYHalf)*5*ff;
        rz:=screen;
        Turn3DRight0(rx,ry,rz,sina,cosa,sinb,cosb,sinc,cosc)
    END
END;

BEGIN
    WITH Camera DO
    BEGIN
        x:=0;
        y:=0;
        z:=0;
        alpha:=0;
        beta:=0;
        gamma:=0;
        sina:=0;
        cosa:=1;
        sinb:=0;
        cosb:=1;
        sinc:=0;
        cosc:=1
    END;
    scrx:=0;
    scry:=0;
    scrz:=screen;

    ff:=1;

    SetFrame(1023,767);

    screen:=screenOrg;

END.
