{$A+,B-,D+,E-,F+,G+,I-,L+,N+,O-,R-,S-,V-}
{$DEFINE PROFILERx}
{$DEFINE Statisticsx}
{$DEFINE IntNamesx}

UNIT Intervalls;

INTERFACE

USES Maths,Basis,Messages,ObjPoints
{$IFDEF PROFILER}
     ,Profiler
{$ENDIF}
     ;

CONST maxBlock=20;


TYPE IntervallSet=OBJECT
         cnt:WORD;
         block:ARRAY[1..maxBlock] OF RECORD
             iFrom,iTo:ObjectPoint
         END;

{$IFDEF IntNames}
         name:STRING20;
{$ENDIF}

         PROCEDURE NotOp(VAR int:IntervallSet);

         PROCEDURE OrOp(VAR int1,int2:IntervallSet);
         PROCEDURE AndOp(VAR int1,int2:IntervallSet);
         PROCEDURE AndNotOp(VAR int1,int2:IntervallSet);

         PROCEDURE OrOpSmooth(VAR int1,int2:IntervallSet;rSmooth:RealType);
         PROCEDURE AndOpSmooth(VAR int1,int2:IntervallSet;rSmooth:RealType);
         PROCEDURE AndNotOpSmooth(VAR int1,int2:IntervallSet;rSmooth:RealType);

         PROCEDURE ReNormal(VAR m:Matrix3);

     END;

PROCEDURE TraceIntervallStats;



IMPLEMENTATION

{$IFDEF Statistics}
TYPE StatAr=ARRAY[0..maxBlock,0..maxBlock] OF LONGINT;
VAR OrOpCnt,AndOpCnt,AndNotOpCnt,OrOpSmoothCnt,AndOpSmoothCnt,AndNotOpSmoothCnt:StatAr;

    statI,statJ:INTEGER;
{$ENDIF}


PROCEDURE IntervallSet.NotOp(VAR int:IntervallSet);
VAR i:INTEGER;
BEGIN
{$IFDEF PROFILER}
     BeginIt(1);
{$ENDIF}

{$IFDEF IntNames}
    name:='-('+int.name+')';
{$ENDIF}
    IF (int.cnt=0) THEN
    BEGIN
        block[1].iFrom:=ninfinitePoint;
        block[1].iTo:=infinitePoint;

        cnt:=1
    END
    ELSE BEGIN
        IF int.block[1].iFrom.l<=-oInfinite THEN
        BEGIN
            FOR i:=1 TO PRED(int.cnt) DO
            BEGIN
                CopyInverse(block[i].iFrom,int.block[i].iTo);
                CopyInverse(block[i].iTo,int.block[SUCC(i)].iFrom)
            END;
            IF int.block[int.cnt].iTo.l>=oInfinite THEN
                cnt:=PRED(int.cnt)
            ELSE BEGIN
                CopyInverse(block[int.cnt].iFrom,int.block[int.cnt].iTo);
                block[int.cnt].iTo:=infinitePoint;

                cnt:=int.cnt
            END
        END
        ELSE BEGIN
            block[1].iFrom:=ninfinitePoint;

            CopyInverse(block[1].iTo,int.block[1].iFrom);

            FOR i:=2 TO int.cnt DO
            BEGIN
                CopyInverse(block[i].iFrom,int.block[PRED(i)].iTo);
                CopyInverse(block[i].iTo,int.block[i].iFrom)
            END;
            IF int.block[int.cnt].iTo.l<oInfinite THEN
            BEGIN
                CopyInverse(block[SUCC(int.cnt)].iFrom,int.block[int.cnt].iTo);
                block[SUCC(int.cnt)].iTo:=infinitePoint;

                IF (int.cnt=maxBlock) THEN
                BEGIN
                  {$IFDEF IntNames}
                    trace.Add('IntervallSet:'+Name+'->BlockOvl');
                  {$ELSE}
                    trace.Add('IntervallSet:BlockOvl');
                  {$ENDIF}
                    cnt:=int.cnt;
                END
                ELSE
                    cnt:=SUCC(int.cnt)
            END
            ELSE
                cnt:=int.cnt
        END
    END
{$IFDEF PROFILER}
     ;EndIt(1);
{$ENDIF}
END;



PROCEDURE IntervallSet.OrOp(VAR int1,int2:IntervallSet);
VAR i1,i2,i3:INTEGER;


    PROCEDURE New1;
    BEGIN
        IF (i3=maxBlock) THEN
         {$IFDEF IntNames}
           trace.Add('IntervallSet:'+Name+'->BlockOvl')
         {$ELSE}
           trace.Add('IntervallSet:BlockOvl')
         {$ENDIF}
        ELSE
            INC(i3);
        block[i3].iFrom:=int1.block[i1].iFrom;
        block[i3].iTo:=int1.block[i1].iTo
    END;

    PROCEDURE New2;
    BEGIN
        IF (i3=maxBlock) THEN
         {$IFDEF IntNames}
           trace.Add('IntervallSet:'+Name+'->BlockOvl')
         {$ELSE}
           trace.Add('IntervallSet:BlockOvl')
         {$ENDIF}
        ELSE
            INC(i3);
        block[i3].iFrom:=int2.block[i2].iFrom;
        block[i3].iTo:=int2.block[i2].iTo
    END;

    PROCEDURE Take1;
    BEGIN
        IF int1.block[i1].iFrom.l>block[i3].iTo.l THEN
            New1
        ELSE IF int1.block[i1].iTo.l>block[i3].iTo.l THEN
            block[i3].iTo:=int1.block[i1].iTo;
        INC(i1);
    END;

    PROCEDURE Take2;
    BEGIN
        IF int2.block[i2].iFrom.l>block[i3].iTo.l THEN
            New2
        ELSE IF int2.block[i2].iTo.l>block[i3].iTo.l THEN
            block[i3].iTo:=int2.block[i2].iTo;
        INC(i2);
    END;

BEGIN
{$IFDEF PROFILER}
     BeginIt(2);
{$ENDIF}
{$IFDEF Statistics}
    INC(OrOpCnt[int1.cnt,int2.cnt]);
{$ENDIF}
{$IFDEF IntNames}
    name:='('+int1.name+'+'+int2.name+')';
{$ENDIF}

    IF int1.cnt=0 THEN
    BEGIN
        FOR i2:=1 TO int2.cnt DO
            block[i2]:=int2.block[i2];
        cnt:=int2.cnt
    END
    ELSE IF int2.cnt=0 THEN
    BEGIN
        FOR i1:=1 TO int1.cnt DO
            block[i1]:=int1.block[i1];
        cnt:=int1.cnt
    END
    ELSE BEGIN

        i1:=1;
        i2:=1;
        i3:=0;

        IF int1.block[i1].iFrom.l<int2.block[i2].iFrom.l THEN
        BEGIN
            New1;
            INC(i1)
        END
        ELSE BEGIN
            New2;
            INC(i2)
        END;

        WHILE (i1<=int1.cnt) OR (i2<=int2.cnt) DO
        BEGIN
            IF (i1>int1.cnt) THEN
                Take2
            ELSE IF (i2>int2.cnt) THEN
                Take1
            ELSE IF int1.block[i1].iFrom.l<int2.block[i2].iFrom.l THEN
                Take1
            ELSE
                Take2
        END;
        cnt:=i3
    END
{$IFDEF PROFILER}
    ;EndIt(2);
{$ENDIF}
END;




PROCEDURE IntervallSet.AndOp(VAR int1,int2:IntervallSet);
VAR int1Inv,int2Inv,resultInv:IntervallSet;
BEGIN
{$IFDEF PROFILER}
     BeginIt(3);
{$ENDIF}
{$IFDEF Statistics}
    INC(AndOpCnt[int1.cnt,int2.cnt]);
{$ENDIF}
{$IFDEF IntNames}
    name:='('+int1.name+'*'+int2.name+')';
{$ENDIF}
    IF (int1.cnt=0) OR (int2.cnt=0) THEN
    BEGIN
        cnt:=0
    END
    ELSE IF (int1.cnt=1) AND (int2.cnt=1) THEN
    BEGIN
        IF (int1.block[1].iTo.l<int2.block[1].iFrom.l) OR
           (int1.block[1].iFrom.l>int2.block[1].iTo.l)
        THEN
            cnt:=0
        ELSE BEGIN
            cnt:=1;
            IF int1.block[1].iFrom.l>int2.block[1].iFrom.l THEN
                block[1].iFrom:=int1.block[1].iFrom
            ELSE
                block[1].iFrom:=int2.block[1].iFrom;
            IF int1.block[1].iTo.l<int2.block[1].iTo.l THEN
                block[1].iTo:=int1.block[1].iTo
            ELSE
                block[1].iTo:=int2.block[1].iTo
        END
    END
    ELSE BEGIN
        int1Inv.NotOp(int1);
        int2Inv.NotOp(int2);
        resultInv.OrOp(int1Inv,int2Inv);
        NotOp(resultInv)
    END

{$IFDEF PROFILER}
    ;EndIt(3);
{$ENDIF}
END;


PROCEDURE IntervallSet.AndNotOp(VAR int1,int2:IntervallSet);
VAR int1Inv,resultInv:IntervallSet;
    i1:INTEGER;
BEGIN
{$IFDEF PROFILER}
     BeginIt(4);
{$ENDIF}
{$IFDEF Statistics}
    INC(AndNotOpCnt[int1.cnt,int2.cnt]);
{$ENDIF}
{$IFDEF IntNames}
    name:='('+int1.name+'*-'+int2.name+')';
{$ENDIF}

   IF (int2.cnt=0) OR
      ((int1.cnt=1) AND (int2.cnt=1) AND
       ((int2.block[1].iFrom.l>int1.block[1].iTo.l) OR
        (int2.block[1].iTo.l<int1.block[1].iFrom.l)
       )
      ) THEN
    BEGIN
        FOR i1:=1 TO int1.cnt DO
            block[i1]:=int1.block[i1];
        cnt:=int1.cnt
    END
    ELSE BEGIN
        int1Inv.NotOp(int1);
        resultInv.OrOp(int1Inv,int2);
        NotOp(resultInv)
    END;
{$IFDEF PROFILER}
     ;EndIt(4);
{$ENDIF}
END;


PROCEDURE IntervallSet.OrOpSmooth(VAR int1,int2:IntervallSet;rSmooth:RealType);
VAR i1,i2,i3:INTEGER;


    PROCEDURE AdaptFkt(l1,l2:RealType;VAR f1,f2:RealType);
    BEGIN
        f1:=0.5+ABS(l1-l2)*rSmooth;
        IF f1>1 THEN
            f1:=1;
        f2:=(1-f1)
    END;



    PROCEDURE Adapt(VAR p1,p2:ObjectPoint);
    VAR f1,f2:RealType;
        nx,ny,nz:RealType;
    BEGIN
        AdaptFkt(p1.l,p2.l,f1,f2);

        nx:=p1.n.x*f1+p2.n.x*f2;
        ny:=p1.n.y*f1+p2.n.y*f2;
        nz:=p1.n.z*f1+p2.n.z*f2;
        p2.n.x:=p2.n.x*f1+p1.n.x*f2;
        p2.n.y:=p2.n.y*f1+p1.n.y*f2;
        p2.n.z:=p2.n.z*f1+p1.n.z*f2;
        p1.n.x:=nx;
        p1.n.y:=ny;
        p1.n.z:=nz;
        VnormSelf(p1.n);
        VnormSelf(p2.n)
    END;

    PROCEDURE Adapt1(VAR p1,p2:ObjectPoint);
    VAR f1,f2:RealType;
    BEGIN
        AdaptFkt(p1.l,p2.l,f1,f2);

        p1.n.x:=p1.n.x*f1+p2.n.x*f2;
        p1.n.y:=p1.n.y*f1+p2.n.y*f2;
        p1.n.z:=p1.n.z*f1+p2.n.z*f2;
        VnormSelf(p1.n)
    END;

    PROCEDURE New1;
    BEGIN
        IF (i3=maxBlock) THEN
         {$IFDEF IntNames}
           trace.Add('IntervallSet:'+Name+'->BlockOvl')
         {$ELSE}
           trace.Add('IntervallSet:BlockOvl')
         {$ENDIF}
        ELSE
            INC(i3);
        block[i3].iFrom:=int1.block[i1].iFrom;
        block[i3].iTo:=int1.block[i1].iTo
    END;

    PROCEDURE New2;
    BEGIN
        IF (i3=maxBlock) THEN
         {$IFDEF IntNames}
           trace.Add('IntervallSet:'+Name+'->BlockOvl')
         {$ELSE}
           trace.Add('IntervallSet:BlockOvl')
         {$ENDIF}
        ELSE
            INC(i3);
        block[i3].iFrom:=int2.block[i2].iFrom;
        block[i3].iTo:=int2.block[i2].iTo
    END;

    PROCEDURE Take1Smooth;
    VAR itox:ObjectPoint;
    BEGIN
        IF int1.block[i1].iFrom.l>block[i3].iTo.l THEN
        BEGIN
            New1;
            Adapt(block[PRED(i3)].iTo,block[i3].iFrom)
        END
        ELSE BEGIN
            IF int1.block[i1].iTo.l>block[i3].iTo.l THEN
            BEGIN
                itox:=block[i3].iTo;
                block[i3].iTo:=int1.block[i1].iTo;
                Adapt1(block[i3].iTo,itox)
            END
            ELSE
                Adapt1(block[i3].iTo,int1.block[i1].iTo);
            Adapt1(block[i3].iFrom,int1.block[i1].iFrom);
        END;
        INC(i1);
    END;

    PROCEDURE Take2Smooth;
    VAR itox:ObjectPoint;
    BEGIN
        IF int2.block[i2].iFrom.l>block[i3].iTo.l THEN
        BEGIN
            New2;
            Adapt(block[PRED(i3)].iTo,block[i3].iFrom)
        END
        ELSE BEGIN
            IF int2.block[i2].iTo.l>block[i3].iTo.l THEN
            BEGIN
                itox:=block[i3].iTo;
                block[i3].iTo:=int2.block[i2].iTo;
                Adapt1(block[i3].iTo,itox)
            END
            ELSE
                Adapt1(block[i3].iTo,int2.block[i2].iTo);
            Adapt1(block[i3].iFrom,int2.block[i2].iFrom);
        END;
        INC(i2);                                              
    END;

BEGIN
{$IFDEF Statistics}
    INC(OrOpSmoothCnt[int1.cnt,int2.cnt]);
{$ENDIF}
{$IFDEF IntNames}
    name:='('+int1.name+'+~'+int2.name+')';
{$ENDIF}
    IF int1.cnt=0 THEN
    BEGIN
        FOR i2:=1 TO int2.cnt DO
            block[i2]:=int2.block[i2];
        cnt:=int2.cnt
    END
    ELSE IF int2.cnt=0 THEN
    BEGIN
        FOR i1:=1 TO int1.cnt DO
            block[i1]:=int1.block[i1];
        cnt:=int1.cnt
    END
    ELSE BEGIN

        i1:=1;
        i2:=1;
        i3:=0;

        IF int1.block[i1].iFrom.l<int2.block[i2].iFrom.l THEN
        BEGIN
            New1;
            INC(i1)
        END
        ELSE BEGIN
            New2;
            INC(i2)
        END;

        WHILE (i1<=int1.cnt) OR (i2<=int2.cnt) DO
        BEGIN
            IF (i1>int1.cnt) THEN
                Take2Smooth
            ELSE IF (i2>int2.cnt) THEN
                Take1Smooth
            ELSE IF int1.block[i1].iFrom.l<int2.block[i2].iFrom.l THEN
                Take1Smooth
            ELSE
                Take2Smooth
        END;
        cnt:=i3
    END
END;


PROCEDURE IntervallSet.AndOpSmooth(VAR int1,int2:IntervallSet;rSmooth:RealType);
VAR int1Inv,int2Inv,resultInv:IntervallSet;
BEGIN
{$IFDEF Statistics}
    INC(AndOpSmoothCnt[int1.cnt,int2.cnt]);
{$ENDIF}
{$IFDEF IntNames}
    name:='('+int1.name+'*~'+int2.name+')';
{$ENDIF}
    int1Inv.NotOp(int1);
    int2Inv.NotOp(int2);
    resultInv.OrOpSmooth(int1Inv,int2Inv,rSmooth);
    NotOp(resultInv);
END;


PROCEDURE IntervallSet.AndNotOpSmooth(VAR int1,int2:IntervallSet;rSmooth:RealType);
VAR int1Inv,resultInv:IntervallSet;
BEGIN
{$IFDEF Statistics}
    INC(AndNotOpSmoothCnt[int1.cnt,int2.cnt]);
{$ENDIF}
{$IFDEF IntNames}
    name:='('+int1.name+'*~-'+int2.name+')';
{$ENDIF}
    int1Inv.NotOp(int1);
    resultInv.OrOpSmooth(int1Inv,int2,rSmooth);
    NotOp(resultInv);
END;

PROCEDURE IntervallSet.ReNormal(VAR m:Matrix3);
VAR i:WORD;
BEGIN
    FOR i:=1 TO cnt DO
    BEGIN
        WITH block[i] DO
        BEGIN
            WITH iFrom DO
            BEGIN
                IF ABS(l)<oInfinite THEN
                    MVmulSelf(m,n)
            END;
            WITH iTo DO
            BEGIN
                IF ABS(l)<oInfinite THEN
                    MVmulSelf(m,n)
            END
        END
    END
END;

{$IFDEF Statistics}

PROCEDURE TraceIntervallStats;
    PROCEDURE TraceStatAr(s:STRING;VAR cntAr:statAr);
    VAR i,j:INTEGER;
        line:STRING;
        item:STRING;
        sum:RealType;
    BEGIN
        sum:=0;
        FOR i:=0 TO maxBlock DO
            FOR j:=0 TO maxBlock DO
                sum:=sum+cntAr[i,j];
        trace.Add(s);
        IF sum=0 THEN
        BEGIN
            trace.Add('no events')
        END
        ELSE BEGIN
            Str(sum:10:0,item);
            line:=item+' events';
            trace.Add(line);

            line:='    ';
            FOR i:=0 TO maxBlock DO
            BEGIN
                Str(i:6,item);
                line:=line+item
            END;
            trace.Add(line);
            FOR i:=0 TO maxBlock DO
            BEGIN
                Str(i:4,line);
                FOR j:=0 TO maxBlock DO
                BEGIN
                    IF cntAr[i,j]=0 THEN
                        item:='      '
                    ELSE
                        Str(cntAr[i,j]/sum*100:6:1,item);
                    line:=line+item
                END;
                trace.Add(line)
            END
        END;
        trace.Add('')
    END;

BEGIN
    trace.Add('Intervall Statistics');
    trace.Add('');
    TraceStatAr('OrOp',OrOpCnt);
    TraceStatAr('AndOp',AndOpCnt);
    TraceStatAr('AndNotOp',AndNotOpCnt);
    TraceStatAr('OrOpSmooth',OrOpSmoothCnt);
    TraceStatAr('AndOpSmooth',AndOpSmoothCnt);
    TraceStatAr('AndNotOpSmooth',AndNotOpSmoothCnt);
    trace.Add('End of Intervall Statistics');
END;

{$ELSE}

PROCEDURE TraceIntervallStats;
BEGIN
END;

{$ENDIF}


BEGIN
{$IFDEF Statistics}
   FOR statI:=0 TO maxBlock DO
       FOR statJ:=0 TO maxBlock DO
       BEGIN
           OrOpCnt[statI,statJ]:=0;
           AndOpCnt[statI,statJ]:=0;
           AndNotOpCnt[statI,statJ]:=0;
           OrOpSmoothCnt[statI,statJ]:=0;
           AndOpSmoothCnt[statI,statJ]:=0;
           AndNotOpSmoothCnt[statI,statJ]:=0
       END
{$ENDIF}
END.
