{$A+,B-,D+,E-,F+,G+,I-,L+,N+,O-,R-,S-,V-}
{$DEFINE ObjNamesx}

UNIT Models;

INTERFACE

USES Maths,SolidObjects;


PROCEDURE Sphere(VAR obj:SolidObject;r:RealType);
(*** Kugel mit Radius r                               ***)

PROCEDURE Beam(VAR obj:SolidObject;r:RealType);
(*** Unendlich ausgedehnter Zylinder in y-Richtung    ***)
(*** mit Radius r                                     ***)

PROCEDURE Wall(VAR obj:SolidObject;r:RealType);
(*** Unendlich ausgedehnte Doppelwand in x-z-Richtung ***)
(*** mit Abstand r von der 0-Ebene                    ***)
(*** (Auenflchen zeigen in den Freiraum)            ***)

PROCEDURE SingleWall(VAR obj:SolidObject;r:RealType);
(*** Unendlich ausgedehnte Ebene in x-z-Richtung mit  ***)
(*** Hhe r ber der 0-Ebene                          ***)
(*** (Auenflche ist oben)                           ***)

PROCEDURE Cone(VAR obj:SolidObject;r:RealType);
(*** Unendlicher Doppelkegel in y-Richtung mit Stei-  ***)
(*** gerungsfaktor r                                  ***)

PROCEDURE Hat(VAR obj:SolidObject;r:RealType);
(*** Hut (Parabel rotiert) in y-Richtung mit Stei-    ***)
(*** gerungsfaktor r                                  ***)

IMPLEMENTATION

PROCEDURE Sphere(VAR obj:SolidObject;r:RealType);
VAR s:STRING;
BEGIN
    WITH obj DO
    BEGIN
        f1:=1;f2:=1;f3:=1;
        f4:=0;f5:=0;f6:=0;
        fr:=Sqr(r);
        Str(r:4:0,s);
{$IFDEF ObjNames}
        name:='Sph['+s+']'
{$ENDIF}
    END
END;


PROCEDURE Beam(VAR obj:SolidObject;r:RealType);
VAR s:STRING;
BEGIN
    WITH obj DO
    BEGIN
        f1:=1;f2:=0;f3:=1;
        f4:=0;f5:=0;f6:=0;
        fr:=Sqr(r);
        Str(r:4:0,s);
{$IFDEF ObjNames}
        name:='Bea['+s+']'
{$ENDIF}
    END
END;


PROCEDURE Wall(VAR obj:SolidObject;r:RealType);
VAR s:STRING;
BEGIN
    WITH obj DO
    BEGIN
        f1:=0;f2:=1;f3:=0;
        f4:=0;f5:=0;f6:=0;
        fr:=Sqr(r);
        Str(r:4:0,s);
{$IFDEF ObjNames}
        name:='Wll['+s+']'
{$ENDIF}
    END
END;


PROCEDURE SingleWall(VAR obj:SolidObject;r:RealType);
VAR s:STRING;
BEGIN
    WITH obj DO
    BEGIN
        f1:=0;f2:=0;f3:=0;
        f4:=0;f5:=1;f6:=0;
        fr:=r;
        Str(r:4:0,s);
{$IFDEF ObjNames}
        name:='Wll['+s+']'
{$ENDIF}
    END
END;


PROCEDURE Cone(VAR obj:SolidObject;r:RealType);
VAR s:STRING;
BEGIN
    WITH obj DO
    BEGIN
        f1:=1;f2:=-r;f3:=1;
        f4:=0;f5:=0;f6:=0;
        fr:=0;
        Str(r:4:0,s);
{$IFDEF ObjNames}
        name:='Con['+s+']'
{$ENDIF}
    END
END;

PROCEDURE Hat(VAR obj:SolidObject;r:RealType);
VAR s:STRING;
BEGIN
    WITH obj DO
    BEGIN
        f1:=1;f2:=0;f3:=1;
        f4:=0;f5:=r;f6:=0;
        fr:=0;
        Str(r:4:0,s);
{$IFDEF ObjNames}
        name:='Hat['+s+']'
{$ENDIF}
    END
END;



END.
